/* $Id: espdr_dfs-test.c,v 1.2 2013-07-26 11:44:44 amodigli Exp $
 *
 * This file is part of the ESPDR Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Author: dsosnows $
 * $Date: 2013-07-26 11:44:44 $
 * $Revision: 1.2 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <cpl.h>

#include <espdr_sigma_clipping.h>
#include <espdr_msg.h>
#include <espdr_led_flat.h>
#include <espdr_utils.h>
/*----------------------------------------------------------------------------*/
/**
 * @defgroup espdr_led_ff  Unit test of espdr_flat
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/

#define INPUT_IMAGE_SIZE    5
#define BASE_VALUE          5000.0

void
espdr_ccd_geom_init(int ext_nb, espdr_CCD_geometry* CCD_geom)
{
    espdr_msg("Filling up the CCD geom structure");

    //strcpy(CCD_geom->instrument, "TEST");
    sprintf(CCD_geom->instrument, "HARPS");
    CCD_geom->ext_nb = ext_nb;
    CCD_geom->total_output_nb = 1;
    CCD_geom->exts = (espdr_CCD_extension*) cpl_malloc(
                    ext_nb * sizeof(espdr_CCD_extension));
    CCD_geom->exts->CCD_nx = 5;
    CCD_geom->exts->CCD_ny = 5;
    CCD_geom->exts->out_nb_x = 1;
    CCD_geom->exts->out_nb_y = 1;
    //CCD_geom->prescan_x = 0;
    //CCD_geom->prescan_y = 0;
    //CCD_geom->overscan_x = 0;
    //CCD_geom->overscan_y = 0;
    //CCD_geom->real_left = 0;
    //CCD_geom->real_right = 5;
    //CCD_geom->real_lower = 0;
    //CCD_geom->real_upper = 5;
    CCD_geom->exts[0].coll_or = 0;
    CCD_geom->exts[0].corr_type = 0;
    CCD_geom->exts[0].rot_angle = 0;
    CCD_geom->exts[0].ver_flip = 0;
    CCD_geom->exts[0].hor_flip = 0;
    CCD_geom->exts[0].pxl_size = 15;
    int out_nb_x = 1;
    int out_nb_y = 1;
    CCD_geom->exts[0].outputs = (espdr_CCD_output**) cpl_malloc(
                    out_nb_x * sizeof(espdr_CCD_output*));
    for (int j = 0; j < out_nb_x; j++) {
        CCD_geom->exts[0].outputs[j] = (espdr_CCD_output*) cpl_malloc(
                        out_nb_y * sizeof(espdr_CCD_output));
    }
    CCD_geom->exts[0].outputs[0][0].real_llx = 1;
    CCD_geom->exts[0].outputs[0][0].real_lly = 1;
    CCD_geom->exts[0].outputs[0][0].real_urx = 5;
    CCD_geom->exts[0].outputs[0][0].real_ury = 5;
    CCD_geom->exts[0].outputs[0][0].oscan_llx = 0;
    CCD_geom->exts[0].outputs[0][0].oscan_lly = 0;
    CCD_geom->exts[0].outputs[0][0].oscan_urx = 0;
    CCD_geom->exts[0].outputs[0][0].oscan_ury = 0;
    CCD_geom->exts[0].outputs[0][0].poscan_llx = 0;
    CCD_geom->exts[0].outputs[0][0].poscan_lly = 0;
    CCD_geom->exts[0].outputs[0][0].poscan_urx = 0;
    CCD_geom->exts[0].outputs[0][0].poscan_ury = 0;
}

void
espdr_led_ff_init(espdr_LED_FF_param* LED_FF_param, char* method_median)
{
    // Fill up the LED_FF parameters structure
    espdr_msg("Filling up the LED_FF parameters structure");
    /*
    LED_FF_param->led_ff_sigma_clipping_method = (char*) cpl_malloc(
                    10 * sizeof(char));
                    */
    //strcpy((char*) LED_FF_param->led_ff_sigma_clipping_method, method_median);
    //sprintf(LED_FF_param->led_ff_sigma_clipping_method, "median");
    //LED_FF_param->led_ff_ksigma = 10.0;
    //LED_FF_param->led_ff_max_iter = 4;
    /*
    LED_FF_param->badpixels_sigma_clipping_method = (char*) cpl_malloc(
                    10 * sizeof(char));
    strcpy((char*) LED_FF_param->badpixels_sigma_clipping_method,
                    method_median);
                    */
    //sprintf(LED_FF_param->badpixels_sigma_clipping_method, "mean");
    /*
    LED_FF_param->badpixels_ksigma = 10.0;
    LED_FF_param->badpixels_max_iter = 10;
    LED_FF_param->gain_sigma_clipping_method = (char*) cpl_malloc(
                    10 * sizeof(char));
    strcpy((char*) LED_FF_param->gain_sigma_clipping_method, method_median);
    */
    //sprintf(LED_FF_param->gain_sigma_clipping_method, "median");
    //LED_FF_param->gain_ksigma = 4.0;
    //LED_FF_param->gain_max_iter = 10;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Unit test of espdr_bad_pixels
 */
/*----------------------------------------------------------------------------*/
/* AMO: Test commented out as CCD geometry structure need to be changed */

static void test_bad_pixels(void)
{
    int i, j;
    cpl_error_code my_error = CPL_ERROR_NONE;

	int static dead_pixel_data[INPUT_IMAGE_SIZE * INPUT_IMAGE_SIZE];
	int static dead_pixel_data_with_dead[INPUT_IMAGE_SIZE * INPUT_IMAGE_SIZE];

    cpl_image *input_image_with_bad_pixels = NULL;
    cpl_image *dead_pixel_image = NULL;
    cpl_image *dead_pixel_image_with_dead = NULL;
    cpl_imagelist *input_imagelist = cpl_imagelist_new();
    cpl_imagelist *input_imagelist_with_bad_pixels = cpl_imagelist_new();
    cpl_imagelist *dead_pixel_imagelist = cpl_imagelist_new();
    cpl_imagelist *dead_pixel_imagelist_with_dead = cpl_imagelist_new();
	double *sigma;
	int *bad_pixels;
	double *mean;
	int ext_nb=1;



    char *method_mean = "mean";
    char *method_median = "median";
    cpl_imagelist *bad_pixels_imagelist = cpl_imagelist_new();
    
    const char *const fctid = "test_bad_pixels";
    const char *const test_subject = "espdr_bad_pixels";
    cpl_errorstate prestate = cpl_errorstate_get();
    
    espdr_CCD_geometry *CCD_geom =
                        (espdr_CCD_geometry *)cpl_malloc( sizeof(espdr_CCD_geometry));
    // Fill up the CCD_geom structure 
    espdr_ccd_geom_init(ext_nb, CCD_geom);
    // Fill up the LED_FF parameters structure 
    espdr_LED_FF_param *LED_FF_param =
                    (espdr_LED_FF_param *)cpl_malloc(sizeof(espdr_LED_FF_param));
    espdr_led_ff_init(LED_FF_param, method_median);
    
    
        // Initialize the test image
    espdr_msg("Initializing input image");
    double test_data[INPUT_IMAGE_SIZE * INPUT_IMAGE_SIZE];
    double test_data_with_bad_pixels[INPUT_IMAGE_SIZE * INPUT_IMAGE_SIZE];
    //cpl_image* input_image = cpl_image_new(5,5,CPL_TYPE_DOUBLE);
    //cpl_image_add_scalar(input_image,5000.);


    for (i = 0; i < INPUT_IMAGE_SIZE*INPUT_IMAGE_SIZE; i++) {
       test_data[i] = BASE_VALUE;
       test_data_with_bad_pixels[i] = BASE_VALUE;
       dead_pixel_data[i] = 0;
       dead_pixel_data_with_dead[i] = 0;
    }

    cpl_image* input_image = cpl_image_wrap_double(INPUT_IMAGE_SIZE, INPUT_IMAGE_SIZE,
                                       test_data);

    //cpl_image_save(input_image, "input_image.fits", CPL_TYPE_FLOAT, NULL, CPL_IO_CREATE);
    espdr_print_rec_status(0);

    my_error = cpl_error_get_code();


    my_error = cpl_imagelist_set(input_imagelist,
                                 cpl_image_duplicate(input_image), 0);
    //cpl_image_delete(input_image);
    dead_pixel_image = cpl_image_wrap_int(INPUT_IMAGE_SIZE, INPUT_IMAGE_SIZE,
                                          dead_pixel_data);
    //cpl_image_save(dead_pixel_image, "dp_image.fits", CPL_BPP_IEEE_FLOAT, NULL, CPL_IO_DEFAULT);

    my_error = cpl_imagelist_set(dead_pixel_imagelist,
                                 cpl_image_duplicate(dead_pixel_image), 0);
    cpl_image_unwrap(dead_pixel_image);
    // Test with invalid input 
    espdr_msg("Test on invalid input");
    
	espdr_bad_pixels(NULL, dead_pixel_imagelist, CCD_geom, LED_FF_param,
                         mean, bad_pixels, sigma,
                         &bad_pixels_imagelist);
	cpl_test_error(CPL_ERROR_NULL_INPUT);

	espdr_bad_pixels(input_imagelist, NULL, CCD_geom, LED_FF_param,
                         mean, bad_pixels, sigma,
                         &bad_pixels_imagelist);
	cpl_test_error(CPL_ERROR_NULL_INPUT);
    
	espdr_bad_pixels(input_imagelist, dead_pixel_imagelist,
                         NULL, LED_FF_param, mean, bad_pixels, sigma,
                         &bad_pixels_imagelist);
	cpl_test_error(CPL_ERROR_NULL_INPUT);

	espdr_bad_pixels(input_imagelist, dead_pixel_imagelist,
                         CCD_geom, NULL, mean, bad_pixels, sigma,
                         &bad_pixels_imagelist);
	cpl_test_error(CPL_ERROR_NULL_INPUT);
    cpl_errorstate_set(prestate);

    // Test with valid input 
    espdr_msg("Test on valid input");
    espdr_msg_warning("Test on valid input");
    {

        mean = (double *)
            cpl_malloc(cpl_imagelist_get_size(input_imagelist)*sizeof(double *));
        sigma = (double *)
            cpl_malloc(cpl_imagelist_get_size(input_imagelist)*sizeof(double *));
        bad_pixels = (int *)
            cpl_malloc(cpl_imagelist_get_size(input_imagelist)*sizeof(int *));

        espdr_bad_pixels(input_imagelist, dead_pixel_imagelist,
                             CCD_geom, LED_FF_param, mean, bad_pixels, sigma,
                             &bad_pixels_imagelist);

        cpl_test_eq(mean[0], 5000.0);
        cpl_test_eq(bad_pixels[0], 0);
        cpl_test_eq(sigma[0], 0.0);
        
        
        test_data_with_bad_pixels[2] = 100.0;
        test_data_with_bad_pixels[15] = 1000.0;
        test_data_with_bad_pixels[20] = 65000.0;

        input_image_with_bad_pixels = cpl_image_wrap_double(INPUT_IMAGE_SIZE,
                                                            INPUT_IMAGE_SIZE,
                                                    test_data_with_bad_pixels);
        
        my_error = cpl_imagelist_set(input_imagelist_with_bad_pixels,
                            cpl_image_duplicate(input_image_with_bad_pixels),
                                     0);
        espdr_msg("error code: %d", my_error);
        
        dead_pixel_data_with_dead[2] = 1;
        dead_pixel_data_with_dead[15] = 1;
        dead_pixel_data_with_dead[20] = 1;
        dead_pixel_image_with_dead = cpl_image_wrap_int(INPUT_IMAGE_SIZE,
                                                        INPUT_IMAGE_SIZE,
                                                        dead_pixel_data_with_dead);
        
        my_error = cpl_imagelist_set(dead_pixel_imagelist_with_dead,
                                cpl_image_duplicate(dead_pixel_image_with_dead),
                                     0);
        espdr_msg("error code: %d", my_error);
        

        espdr_msg_warning("mean image with cpl %g",cpl_image_get_mean(input_image_with_bad_pixels));
	espdr_inst_config *inst_config=NULL;
       espdr_bad_pixels(input_imagelist_with_bad_pixels,
                            dead_pixel_imagelist_with_dead,
			CCD_geom, inst_config, mean, bad_pixels, sigma,
                            &bad_pixels_imagelist);
        
        cpl_test_eq(mean[0], BASE_VALUE);
        cpl_test_eq(bad_pixels[0], 3);
        cpl_test_abs(sigma[0], 0.0, 0.001);
        
        cpl_free(mean);
        cpl_free(sigma);
        cpl_free(bad_pixels);
        cpl_image_unwrap(input_image_with_bad_pixels);
        cpl_imagelist_delete(input_imagelist_with_bad_pixels);
        cpl_image_unwrap(dead_pixel_image_with_dead);
        cpl_imagelist_delete(dead_pixel_imagelist_with_dead);
    }

    for (j = 0; j < 1; j++) {
        cpl_free(CCD_geom->exts[0].outputs[j]);
    }

    cpl_free(CCD_geom->exts[0].outputs);
    cpl_free(CCD_geom->exts);
    cpl_free(CCD_geom->instrument);

    //cpl_free(CCD_geom);

    //cpl_free(LED_FF_param->badpixels_sigma_clipping_method);
    //cpl_free(LED_FF_param->gain_sigma_clipping_method);

    //cpl_free(LED_FF_param->led_ff_sigma_clipping_method);
    cpl_free(LED_FF_param);
    cpl_imagelist_delete(input_imagelist);
    cpl_imagelist_delete(bad_pixels_imagelist);
    cpl_imagelist_delete(dead_pixel_imagelist);
    cpl_image_unwrap(input_image);
    
    cpl_end();
    
    return;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Unit tests of espdr_dfs module
 */
/*----------------------------------------------------------------------------*/

int main(void) {
    cpl_image* ima=NULL;
    int sx=100;
    int sy=100;
    double value=100;
    cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);

    ima = cpl_image_new(sx,sy,CPL_TYPE_DOUBLE);
    cpl_image_add_scalar(ima,value);
    //cpl_image_dump_structure(input_image,stdout);
    //cpl_image_save(ima, "ima.fits", CPL_TYPE_FLOAT, NULL, CPL_IO_DEFAULT);

    //test_bad_pixels();
    cpl_image_delete(ima);
    cpl_test_error(CPL_ERROR_NONE);
    return cpl_test_end(0);
}

/**@}*/
